<?php

namespace FiloBlu\ApiExtension\Model;

use FiloBlu\ApiExtension\Api\GiftCardServiceInterface;
use InvalidArgumentException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\GiftCardAccount\Api\Exception\TooManyAttemptsException;
use Magento\GiftCardAccount\Model\Spi\GiftCardAccountManagerInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 *
 */
class GiftCardService implements GiftCardServiceInterface
{
    /**
     * @var GiftCardAccountManagerInterface
     */
    protected $manager;
    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;


    /**
     * @param GiftCardAccountManagerInterface $manager
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        GiftCardAccountManagerInterface $manager,
        StoreManagerInterface $storeManager
    )
    {
        $this->manager = $manager;
        $this->storeManager = $storeManager;
    }

    /**
     * @param string $giftCardCode
     * @return array[]|string[]
     * @throws LocalizedException
     * @throws NoSuchEntityException
     * @throws TooManyAttemptsException
     */
    public function getGiftCardBalance(string $giftCardCode): array
    {
        return ["data" =>
            [
                "balance" => (float)$this->getBalance($giftCardCode)
            ]
        ];
    }

    /**
     * @param string $giftCardCode
     * @return float
     * @throws NoSuchEntityException
     * @throws LocalizedException
     * @throws TooManyAttemptsException
     */
    private function getBalance(string $giftCardCode): float
    {
        try {
            $giftCard = $this->manager->requestByCode(
                $giftCardCode ,
                (int)$this->storeManager->getWebsite()->getId() ,
                null ,
                true ,
                true
            );
        } catch (NoSuchEntityException $noSuchEntityException) {
            throw new NoSuchEntityException(
                __('This gift card does not exists. Verify and try again.'),
                $noSuchEntityException
            );
        } catch (InvalidArgumentException $e) {
            throw new NoSuchEntityException(
                __("The gift card code is either incorrect or expired. Verify and try again."),
                $e
            );
        }

        return $giftCard->getBalance();
    }
}
