<?php

declare(strict_types=1);

namespace FiloBlu\ApiExtension\Model;

use FiloBlu\ApiExtension\Api\Data\TrackingInterface;
use FiloBlu\ApiExtension\Api\Data\TrackingInterfaceFactory;
use FiloBlu\ApiExtension\Api\OrderServiceInterface;
use FiloBlu\ShipmentEmailTrack\Helper\Data;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Api\ShipmentRepositoryInterface;
use Magento\Sales\Model\Order;

/**
 *
 */
class OrderService implements OrderServiceInterface
{
    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface
     */
    private $orderRepository;
    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;
    /**
     * @var \FiloBlu\ShipmentEmailTrack\Helper\Data
     */
    private $helper;
    /**
     * @var \Magento\Sales\Api\ShipmentRepositoryInterface
     */
    private $shipmentRepository;
    /**
     * @var \FiloBlu\ApiExtension\Api\Data\TrackingInterfaceFactory
     */
    private $trackingFactory;

    /**
     * @param \FiloBlu\ShipmentEmailTrack\Helper\Data $helper
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Sales\Api\ShipmentRepositoryInterface $shipmentRepository
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \FiloBlu\ApiExtension\Api\Data\TrackingInterfaceFactory $trackingFactory
     */
    public function __construct(
        Data $helper,
        OrderRepositoryInterface $orderRepository,
        ShipmentRepositoryInterface $shipmentRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        TrackingInterfaceFactory $trackingFactory
    ) {
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->helper = $helper;
        $this->shipmentRepository = $shipmentRepository;
        $this->trackingFactory = $trackingFactory;
    }

    /**
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getTrackingList(string $orderIncrementId): array
    {
        $result = $this->shipmentRepository->getList(
            $this->searchCriteriaBuilder->addFilter('order_increment_id', $orderIncrementId)->create()
        );

        if ($result->getTotalCount() === 0) {
            throw new NoSuchEntityException(__('No shipment(s) found'));
        }

        $output = [];
        foreach ($result->getItems() as $shipment) {
            foreach ($shipment->getAllTracks() as $track) {
                $output[] = $this->trackingFactory->create([
                    'data' => [
                        TrackingInterface::CARRIER         => $track->getTitle(),
                        TrackingInterface::TRACKING_URL    => $this->helper->getTrackingNumberLink($track),
                        TrackingInterface::TRACKING_NUMBER => $track->getNumber()
                    ]
                ]);
            }
        }
        return $output;
    }
}